'
' * Created by toText.net.
' * Date: 10/02/2009
' 

Imports System.Net
Imports System.Text
Imports System.IO
Imports System.Threading

''' <summary>
''' A class to perform HTTP Posts to toText.net
''' </summary>
Public Class HTTPPoster
	#Region "Private Member Variables"
	''' <summary>The webpage to request as defined the toText.net integration guide</summary>
	Private Const SEND_API_STRING As String = "http://www.totext.net/selfcare/com/zipzaptext/selfcare/SendMessagesApi.action"

	''' <summary>API username associated with the account</summary>
	Private mUsername As String = ""

	''' <summary>API password associated with the account</summary>
	Private mPassword As String = ""

	''' <summary>The mobile phone number to send the message to (in string form)</summary>
	Private mDestination As String = ""

	''' <summary>The message to send</summary>
	Private mMessage As String = ""

	''' <summary>Whether to send the message in test mode</summary>
	Private mTestMode As System.Nullable(Of Boolean) = Nothing

	#End Region

	#Region "Constructor"

	''' <summary>
	''' Constructor for the class
	''' </summary>
	''' <param name="username">API username associated with the account</param>
	''' <param name="password">API password associated with the account</param>
	''' <param name="destination">The mobile phone number to send the message to (in string form)</param>
	''' <param name="message">The message to send</param>
	''' <param name="testMode">If true, sends the message in TestMode</param>
	Public Sub New(username As String, password As String, destination As String, message As String, testMode As System.Nullable(Of Boolean))
		mUsername = username
		mPassword = password
		mDestination = destination
		mMessage = message
		mTestMode = testMode
	End Sub

	#End Region

	#Region "Public Methods"
	''' <summary>
	''' Perform the HTTP Post
	''' </summary>
	''' <returns>A response string returned as a result of the post</returns>
	Public Function Post() As String

		Dim returnString As String = ""

		' prepare the web page we will be asking for
		Dim buffer As Byte() = Encoding.ASCII.GetBytes(CreateRequestString())
		Dim request As HttpWebRequest = CreateAndInitialiseRequest(buffer.Length)
		Dim PostData As Stream = request.GetRequestStream()
		'Now we write, and afterwards, we close. Closing is always important!
		PostData.Write(buffer, 0, buffer.Length)
		PostData.Close()

		Dim response As HttpWebResponse = DirectCast(request.GetResponse(), HttpWebResponse)

		' we will read data via the response stream
		Dim resStream As Stream = response.GetResponseStream()

		returnString = GetStringFromResponseStream(resStream)

		response.Close()

		Return returnString

	End Function

	#End Region

	#Region "Private Methods"

	''' <summary>
	''' Create and init an HTTPWebRequest to make a post with
	''' </summary>
	''' <param name="contentLength">The length of the POST data to follow</param>
	''' <returns>An initialised HTTPWebRequest</returns>
	Private Function CreateAndInitialiseRequest(contentLength As Integer) As HttpWebRequest
		' prepare the web page we will be asking for
		Dim request As HttpWebRequest = DirectCast(WebRequest.Create(SEND_API_STRING), HttpWebRequest)

		request.Credentials = CredentialCache.DefaultCredentials
		request.Method = "POST"
		request.ContentType = "application/x-www-form-urlencoded"
		request.ContentLength = contentLength
		Return request
	End Function

	''' <summary>
	''' Extracts a string from a stream
	''' </summary>
	''' <param name="resStream">The stream to extract from</param>
	''' <returns>The string extracted</returns>
	Private Shared Function GetStringFromResponseStream(resStream As Stream) As String

		Dim tempString As String = Nothing
		Dim count As Integer = 0

		' used to extract the string
		Dim sb As New StringBuilder()

		' used on each read operation
		Dim buf As Byte() = New Byte(8191) {}

		Do
			' fill the buffer with data
			count = resStream.Read(buf, 0, buf.Length)

			' make sure we read some data
			If count <> 0 Then
				' translate from bytes to ASCII text
				tempString = Encoding.ASCII.GetString(buf, 0, count)

				' continue building the string
				sb.Append(tempString)
			End If
		Loop While count > 0
		' any more data to read?
		Return sb.ToString()
	End Function

	''' <summary>
	''' Generate a string of data to post from the construction parameters
	''' </summary>
	''' <returns>The string to post</returns>
	Private Function CreateRequestString() As String
		Dim returnString As String = String.Format("username={0}&password={1}&destination={2}&message={3}", mUsername, mPassword, mDestination, mMessage)
		If mTestMode.HasValue Then
			returnString = String.Format("{0}&testMode={1}", returnString, If(CBool(mTestMode), 1, 0))
		End If
		Return returnString

	End Function

	#End Region
End Class
